@@ -38,25 +38,22 @@
 #include "ui_prefs_project_local.h"
 
 
-#include <kconfigdialog.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
-#include <klocale.h>
-#include <kicon.h>
-#include <kstatusbar.h>
-#include <kdebug.h>
-
-#include <kurl.h>
-#include <kfiledialog.h>
+#include <klocalizedstring.h>
 #include <kmessagebox.h>
-
+#include <keditlistwidget.h>
+#include <kconfigdialog.h>
 #include <kross/core/manager.h>
 #include <kross/core/actioncollection.h>
 #include <kross/ui/model.h>
-#include <threadweaver/ThreadWeaver.h>
+
+#include <QIcon>
 #include <QBoxLayout>
+#include <QFileDialog>
 #include <QDragEnterEvent>
 #include <QDropEvent>
+#include <QTimer>
+#include <QMimeData>
+#include <QDebug>
 
 //#include <sonnet/configwidget.h>
 
@@ -105,7 +102,8 @@ void SettingsController::showSettingsDialog()
     KConfigGroup grp = Settings::self()->config()->group("Identity");
 
     ui_prefs_identity.DefaultLangCode->setModel(LanguageListModel::instance()->sortModel());
-    ui_prefs_identity.DefaultLangCode->setCurrentIndex(LanguageListModel::instance()->sortModelRowForLangCode( grp.readEntry("DefaultLangCode",KGlobal::locale()->language()) ));
+    ui_prefs_identity.DefaultLangCode->setCurrentIndex(LanguageListModel::instance()->sortModelRowForLangCode( grp.readEntry("DefaultLangCode",
+                                                                                                                             QLocale::system().name()) ));
 
     connect(ui_prefs_identity.DefaultLangCode,SIGNAL(activated(int)),ui_prefs_identity.kcfg_DefaultLangCode,SLOT(setLangCode(int)));
     ui_prefs_identity.kcfg_DefaultLangCode->hide();
@@ -184,7 +182,7 @@ bool SettingsController::ensureProjectIsLoaded()
         return true;
 
     int answer=KMessageBox::questionYesNoCancel(m_mainWindowPtr, i18n("You have accessed a feature that requires a project to be loaded. Do you want to create a new project or open an existing project?"),
-        QString(), KGuiItem(i18nc("@action","New"),KIcon("document-new")), KGuiItem(i18nc("@action","Open"),KIcon("project-open"))
+        QString(), KGuiItem(i18nc("@action","New"),QIcon::fromTheme("document-new")), KGuiItem(i18nc("@action","Open"),QIcon::fromTheme("project-open"))
     );
     if (answer==KMessageBox::Yes)
         return projectCreate();
@@ -197,11 +195,11 @@ QString SettingsController::projectOpen(QString path, bool doOpen)
 {
     if (path.isEmpty())
     {
-        Project::instance()->model()->weaver()->suspend();
-        path=KFileDialog::getOpenFileName(KUrl()/*_catalog->url().directory()*/,
-                                          i18n("*.lokalize *.ktp|Lokalize translation project")/*"text/x-lokalize-project"*/,
-                                          m_mainWindowPtr);
-        Project::instance()->model()->weaver()->resume();
+        //Project::instance()->model()->weaver()->suspend();
+        //KDE5PORT mutex if needed
+        path=QFileDialog::getOpenFileName(m_mainWindowPtr, QString(), QDir::homePath()/*_catalog->url().directory()*/,
+                                          i18n("Lokalize translation project (*.lokalize)")/*"text/x-lokalize-project"*/);
+        //Project::instance()->model()->weaver()->resume();
     }
 
     if (!path.isEmpty() && doOpen)
@@ -212,12 +210,13 @@ QString SettingsController::projectOpen(QString path, bool doOpen)
 
 bool SettingsController::projectCreate()
 {
-    Project::instance()->model()->weaver()->suspend();
+    //Project::instance()->model()->weaver()->suspend();
+    //KDE5PORT mutex if needed
     QString desirablePath=Project::instance()->desirablePath();
     if (desirablePath.isEmpty())
         desirablePath=QDir::homePath()+"/index.lokalize";
-    QString path=KFileDialog::getSaveFileName(KUrl(desirablePath), i18n("*.lokalize|Lokalize translation project") /*"text/x-lokalize-project"*/,m_mainWindowPtr);
-    Project::instance()->model()->weaver()->resume();
+    QString path=QFileDialog::getSaveFileName(m_mainWindowPtr, i18nc("@window:title", "Select folder with Gettext .po files to translate"), desirablePath, i18n("Lokalize translation project (*.lokalize)") /*"text/x-lokalize-project"*/);
+    //Project::instance()->model()->weaver()->resume();
     if (path.isEmpty())
         return false;
 
@@ -229,8 +228,12 @@ bool SettingsController::projectCreate()
     }
 
     //TODO ask-n-save
-    Project::instance()->load(path);
-    Project::instance()->setDefaults(); //NOTE will this be an obstacle?
+    QDir projectFolder=QFileInfo(path).absoluteDir();
+    QString projectId=projectFolder.dirName();
+    if (projectFolder.cdUp()) projectId=projectFolder.dirName()%'-'%projectId;;
+    Project::instance()->load(path, QString(), projectId);
+    //Project::instance()->setDefaults(); //NOTE will this be an obstacle?
+    //Project::instance()->setProjectID();
 
     QTimer::singleShot(500, this, SLOT(projectConfigure()));
     return true;
@@ -238,6 +241,12 @@ bool SettingsController::projectCreate()
 
 void SettingsController::projectConfigure()
 {
+    if (Project::instance()->path().isEmpty())
+    {
+        KMessageBox::error(mainWindowPtr(), i18n("Create software or OpenDocument translation project first"));
+        return;
+    }
+
     if (KConfigDialog::showDialog("project_settings"))
     {
         if (!m_projectActionsView->model())
@@ -322,9 +331,8 @@ void SettingsController::projectConfigure()
     ui_prefs_project_local.setupUi(w);
     dialog->addPage(w, Project::local(), i18nc("@title:tab","Personal"), "preferences-desktop-user");
 
-
-    connect(dialog, SIGNAL(settingsChanged(QString)),Project::instance(), SLOT(populateGlossary()));
-    connect(dialog, SIGNAL(settingsChanged(QString)),Project::instance(), SLOT(populateDirModel()));
+    connect(dialog, SIGNAL(settingsChanged(QString)),Project::instance(), SLOT(reinit()));
+    connect(dialog, SIGNAL(settingsChanged(QString)),Project::instance(), SLOT(save()), Qt::QueuedConnection);
     connect(dialog, SIGNAL(settingsChanged(QString)),TM::DBFilesModel::instance(), SLOT( updateProjectTmIndex()));
     connect(dialog, SIGNAL(settingsChanged(QString)),this, SLOT(reflectProjectConfigChange()));
 
@@ -344,7 +352,7 @@ void SettingsController::reflectRelativePathsHack()
     QString projectDir(Project::instance()->projectDir());
     int i=actionz.size();
     while(--i>=0)
-        actionz[i]=KUrl::relativePath(projectDir,actionz.at(i));
+        actionz[i]=QDir(projectDir).relativeFilePath(actionz.at(i));
     m_scriptsRelPrefWidget->setItems(actionz);
 }
 
@@ -355,11 +363,23 @@ void LangCodeSaver::setLangCode(int index)
 
 void RelPathSaver::setText (const QString& txt)
 {
-    QLineEdit::setText(KUrl::relativePath(Project::instance()->projectDir(),
-                       txt));
+    QLineEdit::setText(QDir(Project::instance()->projectDir()).relativeFilePath(txt));
+}
+
+
+void writeUiState(const char* elementName, const QByteArray& state)
+{
+    KConfig config;
+    KConfigGroup cg(&config,"MainWindow");
+    cg.writeEntry(elementName,state.toBase64());
+}
+
+QByteArray readUiState(const char* elementName)
+{
+    KConfig config;
+    KConfigGroup cg(&config,"MainWindow");
+    return QByteArray::fromBase64( cg.readEntry(elementName, QByteArray()) );
 }
 
 
 
-
-#include "prefs.moc"
